package com.s3ai.corporate_app2.controllers

import com.fasterxml.jackson.dataformat.xml.XmlMapper
import com.s3ai.corporate_app2.CinemaServices
import org.springframework.util.ResourceUtils
import java.io.StringReader
import java.io.StringWriter
import java.util.*
import javax.servlet.http.HttpServletResponse
import javax.xml.transform.TransformerFactory
import javax.xml.transform.stream.StreamResult
import javax.xml.transform.stream.StreamSource


fun <T> deleteInstance(id: String, instanceName: String, jpaService: CinemaServices<T>, response: HttpServletResponse): String? {
    var responseString: String? = null
    try {
        val itemId = UUID.fromString(id)
        val user = jpaService.findById(itemId)
        if (null != user) jpaService.delete(user)
        else {
            response.status = HttpServletResponse.SC_NOT_FOUND
            responseString = "$instanceName was not Found"
        }
    } catch (e: IllegalArgumentException) {
        response.status = HttpServletResponse.SC_BAD_REQUEST
        responseString = e.localizedMessage
    }
    return responseString
}

fun <T> getInstance(id: UUID, jpaService: CinemaServices<T>, response: HttpServletResponse): T? {
    val item = jpaService.findById(id)
    if (null != item) return item
    else response.status = HttpServletResponse.SC_NOT_FOUND
    return null
}

fun XMLTransform(item: Any, resource_name: String): String {
    val xmlMapper = XmlMapper()
    val xslt = StreamSource(ResourceUtils.getFile("classpath:xslt/$resource_name"))
    val xml = StreamSource(StringReader(xmlMapper.writeValueAsString(item)))
    val transformer = TransformerFactory.newInstance().newTransformer(xslt)
    val sw = StringWriter()
    transformer.transform(xml, StreamResult(sw))
    return sw.toString()
}

fun <T> getXSLTransformedInstance(id: UUID, jpaService: CinemaServices<T>, resource_name: String, response: HttpServletResponse): String {
    val item = jpaService.findById(id)
    if (null == item) response.status = HttpServletResponse.SC_NOT_FOUND
    else {
        return XMLTransform(item, resource_name)
    }
    return ""
}
