package com.s3ai.corporate_app2.controllers

import com.fasterxml.jackson.databind.ObjectMapper
import com.s3ai.corporate_app2.Cinema
import com.s3ai.corporate_app2.CinemaService
import org.assertj.core.api.Assertions.assertThat
import org.hamcrest.Matchers
import org.hamcrest.CoreMatchers
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*
import java.lang.reflect.GenericArrayType
import java.util.*


@SpringBootTest
@AutoConfigureMockMvc
class CinemasTest {
    @Autowired
    private lateinit var service: CinemaService

    @Autowired
    private lateinit var controller: CinemasController

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        assertThat(controller).isNotNull
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql")
    fun updateCinemaTest() {
        val cinema = service.findAll().random()
        print(cinema.id)

        cinema.name = UUID.randomUUID().toString()

        mvc.perform(MockMvcRequestBuilders.post("/cinemas/update")
                .contentType(MediaType.APPLICATION_JSON)
                .content(ObjectMapper().writeValueAsString(cinema)))
                .andExpect(status().isFound)
                .andExpect(redirectedUrl("/cinemas/list"))

        val updatedCinema = service.findById(cinema.id!!)
        assertThat(cinema == updatedCinema)
    }

    @Test
    @Throws(Exception::class)
    fun editRedirectToCreateIfIdEmptyTest() {
        mvc.perform(MockMvcRequestBuilders.get("/cinemas/edit"))
                .andExpect(status().isOk)
                .andExpect(model().attribute("action", Matchers.equalTo("Create")))
                .andExpect(model().attribute("cinema", Matchers.notNullValue(Cinema::class.java)))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql")
    fun editPageTest() {
        val randomCinemaId = service.findAll().map { e -> e.id }.distinct().random()
        mvc.perform(MockMvcRequestBuilders.get("/cinemas/edit?id=${randomCinemaId}"))
                .andExpect(status().isOk)
                .andExpect(model().attribute("action", Matchers.equalTo("Edit")))
                .andExpect(model().attribute("cinema", Matchers.notNullValue(Cinema::class.java)))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql")
    fun listPageTest() {
        mvc.perform(MockMvcRequestBuilders.get("/cinemas/list"))
                .andExpect(status().isOk)
                .andExpect(model().attribute("cinemas", Matchers.notNullValue(MutableCollection::class.java)))
                .andExpect(model().attribute("cinemas", CoreMatchers.everyItem(Matchers.notNullValue(Cinema::class.java))))
    }
}
