package com.s3ai.corporate_app2.controllers

import com.fasterxml.jackson.databind.ObjectMapper
import com.s3ai.corporate_app2.Ticket
import com.s3ai.corporate_app2.TicketService
import com.s3ai.corporate_app2.User
import org.assertj.core.api.Assertions.assertThat
import org.hamcrest.CoreMatchers
import org.hamcrest.Matchers
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers
import java.util.*


@SpringBootTest
@AutoConfigureMockMvc
class TicketsTest {
    @Autowired
    private lateinit var service: TicketService

    @Autowired
    private lateinit var controller: TicketsController

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        assertThat(controller).isNotNull
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun updateTicketTest() {
        val ticket = service.findAll().random()
        print(ticket.id)

        ticket.movie = UUID.randomUUID().toString()

        mvc.perform(MockMvcRequestBuilders.post("/tickets/update")
                .contentType(MediaType.APPLICATION_JSON)
                .content(ObjectMapper().writeValueAsString(ticket)))
                .andExpect(MockMvcResultMatchers.status().isFound)
                .andExpect(MockMvcResultMatchers.redirectedUrl("/tickets/list"))

        val updatedTicket = service.findById(ticket.id!!)
        assertThat(ticket == updatedTicket)
    }

    @Test
    @Throws(Exception::class)
    fun editRedirectToCreateIfIdEmptyTest() {
        mvc.perform(MockMvcRequestBuilders.get("/tickets/edit"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andExpect(MockMvcResultMatchers.model().attribute("action", Matchers.equalTo("Create")))
                .andExpect(MockMvcResultMatchers.model().attribute("ticket", Matchers.notNullValue(Ticket::class.java)))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun editPageTest() {
        val randomTicketId = service.findAll().map { e -> e.id }.distinct().random()
        mvc.perform(MockMvcRequestBuilders.get("/tickets/edit?id=${randomTicketId}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andExpect(MockMvcResultMatchers.model().attribute("action", Matchers.equalTo("Edit")))
                .andExpect(MockMvcResultMatchers.model().attribute("ticket", Matchers.notNullValue(Ticket::class.java)))
    }

    @Test
    @Throws(Exception::class)
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun listPageTest() {
        mvc.perform(MockMvcRequestBuilders.get("/tickets/list"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andExpect(MockMvcResultMatchers.model().attribute("tickets", CoreMatchers.everyItem(Matchers.notNullValue(Ticket::class.java))))
    }
}