package com.s3ai.corporate_app2.controllers.rest

import com.s3ai.corporate_app2.TicketService
import org.assertj.core.api.Assertions
import org.hamcrest.MatcherAssert
import org.hamcrest.Matchers
import org.hamcrest.core.IsNot
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers

@SpringBootTest
@AutoConfigureMockMvc
class TicketApiTest {
    @Autowired
    private lateinit var service: TicketService

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        Assertions.assertThat(mvc).isNotNull
        Assertions.assertThat(service).isNotNull
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun fillTest() {
        mvc.perform(MockMvcRequestBuilders.get("/api/tickets/fill"))
                .andDo { e: MvcResult -> println(e.response) }
                .andExpect(MockMvcResultMatchers.status().isOk)
        MatcherAssert.assertThat(service.findAll().size, Matchers.equalTo(401))
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun deleteTest() {
        val randomTicketId = service.findAll().map { e -> e.id }.distinct().random()
        print(randomTicketId)
        mvc.perform(MockMvcRequestBuilders.delete("/api/tickets/delete?id=${randomTicketId}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
        val tickets = service.findAll().map { e -> e.id }.distinct()
        MatcherAssert.assertThat(tickets, IsNot.not(Matchers.hasItem(randomTicketId)))
    }
}