package com.s3ai.corporate_app2.controllers.rest

import com.s3ai.corporate_app2.UserService
import com.s3ai.corporate_app2.TicketService
import org.assertj.core.api.Assertions
import org.hamcrest.MatcherAssert
import org.hamcrest.Matchers
import org.hamcrest.core.IsNot
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers

@SpringBootTest
@AutoConfigureMockMvc
class UserApiTest {
    @Autowired
    private lateinit var service: UserService

    @Autowired
    private lateinit var ticketService: TicketService

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        Assertions.assertThat(mvc).isNotNull
        Assertions.assertThat(service).isNotNull
    }

    @Test
    @Sql("/usersFill.sql")
    fun fillTest() {
        mvc.perform(MockMvcRequestBuilders.get("/api/users/fill"))
                .andDo { e: MvcResult -> println(e.response) }
                .andExpect(MockMvcResultMatchers.status().isOk)
        MatcherAssert.assertThat(service.findAll().size, Matchers.equalTo(401))
    }

    @Test
    @Sql("/usersFill.sql", "/cinemasFill.sql", "/ticketsFill.sql")
    fun deleteTest() {
        val randomUserId = ticketService.findAll().map { e -> e.user?.id }.distinct().random()
        print(randomUserId)
        mvc.perform(MockMvcRequestBuilders.delete("/api/users/delete?id=${randomUserId}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
        val tickets = ticketService.findAll().map { e -> e.user!!.id }.distinct()
        MatcherAssert.assertThat(tickets, IsNot.not(Matchers.hasItem(randomUserId)))
    }
}