package com.s3ai.corporate_app2.controllers.rest.formats

import com.fasterxml.jackson.module.kotlin.readValue
import com.s3ai.corporate_app2.Ticket
import com.s3ai.corporate_app2.TicketService
import com.s3ai.corporate_app2.kotlinJsonMapper
import com.s3ai.corporate_app2.kotlinXmlMapper
import org.assertj.core.api.Assertions
import org.hamcrest.CoreMatchers.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers


@SpringBootTest
@AutoConfigureMockMvc
class TicketsFormattingTests {
    @Autowired
    private lateinit var service: TicketService

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        Assertions.assertThat(mvc).isNotNull
        Assertions.assertThat(service).isNotNull
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun allJSONTest() {
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/json/tickets/all"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val cinemas: List<Ticket> = kotlinJsonMapper.readValue(mvcResult.response.contentAsString)
        assertThat(cinemas.size, equalTo(100))
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun itemJSONTest() {
        val randomTicket = service.findAll().random()
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/json/tickets/item/${randomTicket.id}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val parsedTicket: Ticket = kotlinJsonMapper.readValue(mvcResult.response.contentAsString)
        assertThat(parsedTicket.toString(), equalTo(randomTicket.toString()))
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun allXMLTest() {
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/xml/tickets/all"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val cinemas: List<Ticket> = kotlinXmlMapper.readValue(mvcResult.response.contentAsString)
        assertThat(cinemas.size, equalTo(100))
    }

    @Test
    @Sql("/cinemasFill.sql", "/usersFill.sql", "/ticketsFill.sql")
    fun itemXMLTest() {
        val randomTicket = service.findAll().random()
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/xml/tickets/item/${randomTicket.id}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val parsedTicket: Ticket = kotlinXmlMapper.readValue(mvcResult.response.contentAsString)
        assertThat(parsedTicket.toString(), equalTo(randomTicket.toString()))
    }
}