package com.s3ai.corporate_app2.controllers.rest.formats

import com.fasterxml.jackson.databind.DeserializationFeature
import com.fasterxml.jackson.databind.MapperFeature
import com.fasterxml.jackson.dataformat.xml.JacksonXmlModule
import com.fasterxml.jackson.dataformat.xml.XmlMapper
import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import com.fasterxml.jackson.module.kotlin.readValue
import com.fasterxml.jackson.module.kotlin.registerKotlinModule
import com.s3ai.corporate_app2.*
import com.s3ai.corporate_app2.kotlinJsonMapper
import com.s3ai.corporate_app2.kotlinXmlMapper
import org.assertj.core.api.Assertions
import org.hamcrest.CoreMatchers.equalTo
import org.hamcrest.MatcherAssert.assertThat
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.test.context.jdbc.Sql
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.result.MockMvcResultMatchers


@SpringBootTest
@AutoConfigureMockMvc
class UsersFormattingTests {
    @Autowired
    private lateinit var service: UserService

    @Autowired
    private lateinit var mvc: MockMvc

    @Test
    @Throws(Exception::class)
    fun contextLoads() {
        Assertions.assertThat(mvc).isNotNull
        Assertions.assertThat(service).isNotNull
    }

    @Test
    @Sql("/usersFill.sql")
    fun allJSONTest() {
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/json/users/all"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val cinemas: List<User> = kotlinJsonMapper.readValue(mvcResult.response.contentAsString)
        assertThat(cinemas.size, equalTo(100))
    }

    @Test
    @Sql("/usersFill.sql")
    fun itemJSONTest() {
        val randomUser = service.findAll().random()
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/json/users/item/${randomUser.id}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val parsedUser: User = kotlinJsonMapper.readValue(mvcResult.response.contentAsString)
        assertThat(parsedUser.toString(), equalTo(randomUser.toString()))
    }

    @Test
    @Sql("/usersFill.sql")
    fun allXMLTest() {
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/xml/users/all"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val cinemas: List<User> = kotlinXmlMapper.readValue(mvcResult.response.contentAsString)
        assertThat(cinemas.size, equalTo(100))
    }

    @Test
    @Sql("/usersFill.sql")
    fun itemXMLTest() {
        val randomUser = service.findAll().random()
        val mvcResult = mvc.perform(MockMvcRequestBuilders.get("/api/xml/users/item/${randomUser.id}"))
                .andExpect(MockMvcResultMatchers.status().isOk)
                .andReturn()
        val parsedUser: User = kotlinXmlMapper.readValue(mvcResult.response.contentAsString)
        assertThat(parsedUser.toString(), equalTo(randomUser.toString()))
    }
}